from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.file_type import FileType
from ..models.split import Split
from ..models.tagging_schema import TaggingSchema
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.export_request_col_mapping_type_0 import ExportRequestColMappingType0
    from ..models.filter_params import FilterParams


T = TypeVar("T", bound="ExportRequest")


@_attrs_define
class ExportRequest:
    """
    Attributes:
        all_but (Union[Unset, bool]):  Default: False.
        col_mapping (Union['ExportRequestColMappingType0', None, Unset]):  Default: None.
        compare_to (Union[None, Split, Unset]):
        file_type (Union[Unset, FileType]):
        filter_params (Union[Unset, FilterParams]):
        hf_format (Union[Unset, bool]):  Default: False.
        include_cols (Union[None, Unset, list[str]]):
        map_threshold (Union[Unset, float]):  Default: 0.5.
        tagging_schema (Union[None, TaggingSchema, Unset]):
        task (Union[None, Unset, str]):
    """

    all_but: Union[Unset, bool] = False
    col_mapping: Union["ExportRequestColMappingType0", None, Unset] = None
    compare_to: Union[None, Split, Unset] = UNSET
    file_type: Union[Unset, FileType] = UNSET
    filter_params: Union[Unset, "FilterParams"] = UNSET
    hf_format: Union[Unset, bool] = False
    include_cols: Union[None, Unset, list[str]] = UNSET
    map_threshold: Union[Unset, float] = 0.5
    tagging_schema: Union[None, TaggingSchema, Unset] = UNSET
    task: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.export_request_col_mapping_type_0 import ExportRequestColMappingType0

        all_but = self.all_but

        col_mapping: Union[None, Unset, dict[str, Any]]
        if isinstance(self.col_mapping, Unset):
            col_mapping = UNSET
        elif isinstance(self.col_mapping, ExportRequestColMappingType0):
            col_mapping = self.col_mapping.to_dict()
        else:
            col_mapping = self.col_mapping

        compare_to: Union[None, Unset, str]
        if isinstance(self.compare_to, Unset):
            compare_to = UNSET
        elif isinstance(self.compare_to, Split):
            compare_to = self.compare_to.value
        else:
            compare_to = self.compare_to

        file_type: Union[Unset, str] = UNSET
        if not isinstance(self.file_type, Unset):
            file_type = self.file_type.value

        filter_params: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.filter_params, Unset):
            filter_params = self.filter_params.to_dict()

        hf_format = self.hf_format

        include_cols: Union[None, Unset, list[str]]
        if isinstance(self.include_cols, Unset):
            include_cols = UNSET
        elif isinstance(self.include_cols, list):
            include_cols = self.include_cols

        else:
            include_cols = self.include_cols

        map_threshold = self.map_threshold

        tagging_schema: Union[None, Unset, str]
        if isinstance(self.tagging_schema, Unset):
            tagging_schema = UNSET
        elif isinstance(self.tagging_schema, TaggingSchema):
            tagging_schema = self.tagging_schema.value
        else:
            tagging_schema = self.tagging_schema

        task: Union[None, Unset, str]
        if isinstance(self.task, Unset):
            task = UNSET
        else:
            task = self.task

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if all_but is not UNSET:
            field_dict["all_but"] = all_but
        if col_mapping is not UNSET:
            field_dict["col_mapping"] = col_mapping
        if compare_to is not UNSET:
            field_dict["compare_to"] = compare_to
        if file_type is not UNSET:
            field_dict["file_type"] = file_type
        if filter_params is not UNSET:
            field_dict["filter_params"] = filter_params
        if hf_format is not UNSET:
            field_dict["hf_format"] = hf_format
        if include_cols is not UNSET:
            field_dict["include_cols"] = include_cols
        if map_threshold is not UNSET:
            field_dict["map_threshold"] = map_threshold
        if tagging_schema is not UNSET:
            field_dict["tagging_schema"] = tagging_schema
        if task is not UNSET:
            field_dict["task"] = task

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.export_request_col_mapping_type_0 import ExportRequestColMappingType0
        from ..models.filter_params import FilterParams

        d = src_dict.copy()
        all_but = d.pop("all_but", UNSET)

        def _parse_col_mapping(data: object) -> Union["ExportRequestColMappingType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                col_mapping_type_0 = ExportRequestColMappingType0.from_dict(data)

                return col_mapping_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ExportRequestColMappingType0", None, Unset], data)

        col_mapping = _parse_col_mapping(d.pop("col_mapping", UNSET))

        def _parse_compare_to(data: object) -> Union[None, Split, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                compare_to_type_0 = Split(data)

                return compare_to_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Split, Unset], data)

        compare_to = _parse_compare_to(d.pop("compare_to", UNSET))

        _file_type = d.pop("file_type", UNSET)
        file_type: Union[Unset, FileType]
        if isinstance(_file_type, Unset):
            file_type = UNSET
        else:
            file_type = FileType(_file_type)

        _filter_params = d.pop("filter_params", UNSET)
        filter_params: Union[Unset, FilterParams]
        if isinstance(_filter_params, Unset):
            filter_params = UNSET
        else:
            filter_params = FilterParams.from_dict(_filter_params)

        hf_format = d.pop("hf_format", UNSET)

        def _parse_include_cols(data: object) -> Union[None, Unset, list[str]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                include_cols_type_0 = cast(list[str], data)

                return include_cols_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[str]], data)

        include_cols = _parse_include_cols(d.pop("include_cols", UNSET))

        map_threshold = d.pop("map_threshold", UNSET)

        def _parse_tagging_schema(data: object) -> Union[None, TaggingSchema, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                tagging_schema_type_0 = TaggingSchema(data)

                return tagging_schema_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, TaggingSchema, Unset], data)

        tagging_schema = _parse_tagging_schema(d.pop("tagging_schema", UNSET))

        def _parse_task(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        task = _parse_task(d.pop("task", UNSET))

        export_request = cls(
            all_but=all_but,
            col_mapping=col_mapping,
            compare_to=compare_to,
            file_type=file_type,
            filter_params=filter_params,
            hf_format=hf_format,
            include_cols=include_cols,
            map_threshold=map_threshold,
            tagging_schema=tagging_schema,
            task=task,
        )

        export_request.additional_properties = d
        return export_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class GeneratedScorerTaskResultStatus(str, Enum):
    COMPLETED = "completed"
    FAILED = "failed"
    PENDING = "pending"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.update_dataset_content_request import UpdateDatasetContentRequest
from ...types import UNSET, Response, Unset


def _get_kwargs(
    dataset_id: str, *, body: UpdateDatasetContentRequest, if_match: Union[None, Unset, str] = UNSET
) -> dict[str, Any]:
    headers: dict[str, Any] = {}
    if not isinstance(if_match, Unset):
        headers["If-Match"] = if_match

    _kwargs: dict[str, Any] = {"method": "patch", "url": f"/datasets/{dataset_id}/content"}

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, HTTPValidationError]]:
    if response.status_code == 200:
        response_200 = response.json()
        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, HTTPValidationError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    dataset_id: str,
    *,
    client: AuthenticatedClient,
    body: UpdateDatasetContentRequest,
    if_match: Union[None, Unset, str] = UNSET,
) -> Response[Union[Any, HTTPValidationError]]:
    """Update Dataset Content

     Update the content of a dataset.

    The `index` and `column_name` fields are treated as keys tied to a specific version of the dataset.
    As such, these values are considered immutable identifiers for the dataset's structure.

    For example, if an edit operation changes the name of a column, subsequent edit operations in
    the same request should reference the column using its original name.

    The `If-Match` header is used to ensure that updates are only applied if the client's version of the
    dataset
    matches the server's version. This prevents conflicts from simultaneous updates. The `ETag` header
    in the response
    provides the new version identifier after a successful update.

    Args:
        dataset_id (str):
        if_match (Union[None, Unset, str]): ETag of the dataset as a version identifier.
        body (UpdateDatasetContentRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, HTTPValidationError]]
    """

    kwargs = _get_kwargs(dataset_id=dataset_id, body=body, if_match=if_match)

    response = client.get_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


def sync(
    dataset_id: str,
    *,
    client: AuthenticatedClient,
    body: UpdateDatasetContentRequest,
    if_match: Union[None, Unset, str] = UNSET,
) -> Optional[Union[Any, HTTPValidationError]]:
    """Update Dataset Content

     Update the content of a dataset.

    The `index` and `column_name` fields are treated as keys tied to a specific version of the dataset.
    As such, these values are considered immutable identifiers for the dataset's structure.

    For example, if an edit operation changes the name of a column, subsequent edit operations in
    the same request should reference the column using its original name.

    The `If-Match` header is used to ensure that updates are only applied if the client's version of the
    dataset
    matches the server's version. This prevents conflicts from simultaneous updates. The `ETag` header
    in the response
    provides the new version identifier after a successful update.

    Args:
        dataset_id (str):
        if_match (Union[None, Unset, str]): ETag of the dataset as a version identifier.
        body (UpdateDatasetContentRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, HTTPValidationError]
    """

    return sync_detailed(dataset_id=dataset_id, client=client, body=body, if_match=if_match).parsed


async def asyncio_detailed(
    dataset_id: str,
    *,
    client: AuthenticatedClient,
    body: UpdateDatasetContentRequest,
    if_match: Union[None, Unset, str] = UNSET,
) -> Response[Union[Any, HTTPValidationError]]:
    """Update Dataset Content

     Update the content of a dataset.

    The `index` and `column_name` fields are treated as keys tied to a specific version of the dataset.
    As such, these values are considered immutable identifiers for the dataset's structure.

    For example, if an edit operation changes the name of a column, subsequent edit operations in
    the same request should reference the column using its original name.

    The `If-Match` header is used to ensure that updates are only applied if the client's version of the
    dataset
    matches the server's version. This prevents conflicts from simultaneous updates. The `ETag` header
    in the response
    provides the new version identifier after a successful update.

    Args:
        dataset_id (str):
        if_match (Union[None, Unset, str]): ETag of the dataset as a version identifier.
        body (UpdateDatasetContentRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, HTTPValidationError]]
    """

    kwargs = _get_kwargs(dataset_id=dataset_id, body=body, if_match=if_match)

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    dataset_id: str,
    *,
    client: AuthenticatedClient,
    body: UpdateDatasetContentRequest,
    if_match: Union[None, Unset, str] = UNSET,
) -> Optional[Union[Any, HTTPValidationError]]:
    """Update Dataset Content

     Update the content of a dataset.

    The `index` and `column_name` fields are treated as keys tied to a specific version of the dataset.
    As such, these values are considered immutable identifiers for the dataset's structure.

    For example, if an edit operation changes the name of a column, subsequent edit operations in
    the same request should reference the column using its original name.

    The `If-Match` header is used to ensure that updates are only applied if the client's version of the
    dataset
    matches the server's version. This prevents conflicts from simultaneous updates. The `ETag` header
    in the response
    provides the new version identifier after a successful update.

    Args:
        dataset_id (str):
        if_match (Union[None, Unset, str]): ETag of the dataset as a version identifier.
        body (UpdateDatasetContentRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, HTTPValidationError]
    """

    return (await asyncio_detailed(dataset_id=dataset_id, client=client, body=body, if_match=if_match)).parsed



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.group_label import GroupLabel
from ..models.metric_descriptions import MetricDescriptions
from ..types import UNSET, Unset

T = TypeVar("T", bound="MetricWeight")


@_attrs_define
class MetricWeight:
    """
    Attributes:
        description (Union[MetricDescriptions, None, Unset]):
        group_label (Union[GroupLabel, None, Unset]):
        label (Union[None, Unset, str]):
        weight (Union[Unset, float]):  Default: 0.5.
    """

    description: Union[MetricDescriptions, None, Unset] = UNSET
    group_label: Union[GroupLabel, None, Unset] = UNSET
    label: Union[None, Unset, str] = UNSET
    weight: Union[Unset, float] = 0.5
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        description: Union[None, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET
        elif isinstance(self.description, MetricDescriptions):
            description = self.description.value
        else:
            description = self.description

        group_label: Union[None, Unset, str]
        if isinstance(self.group_label, Unset):
            group_label = UNSET
        elif isinstance(self.group_label, GroupLabel):
            group_label = self.group_label.value
        else:
            group_label = self.group_label

        label: Union[None, Unset, str]
        if isinstance(self.label, Unset):
            label = UNSET
        else:
            label = self.label

        weight = self.weight

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if description is not UNSET:
            field_dict["description"] = description
        if group_label is not UNSET:
            field_dict["group_label"] = group_label
        if label is not UNSET:
            field_dict["label"] = label
        if weight is not UNSET:
            field_dict["weight"] = weight

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_description(data: object) -> Union[MetricDescriptions, None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                description_type_0 = MetricDescriptions(data)

                return description_type_0
            except:  # noqa: E722
                pass
            return cast(Union[MetricDescriptions, None, Unset], data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_group_label(data: object) -> Union[GroupLabel, None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                group_label_type_0 = GroupLabel(data)

                return group_label_type_0
            except:  # noqa: E722
                pass
            return cast(Union[GroupLabel, None, Unset], data)

        group_label = _parse_group_label(d.pop("group_label", UNSET))

        def _parse_label(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        label = _parse_label(d.pop("label", UNSET))

        weight = d.pop("weight", UNSET)

        metric_weight = cls(description=description, group_label=group_label, label=label, weight=weight)

        metric_weight.additional_properties = d
        return metric_weight

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import json


class APIException(Exception):
    """
    APIException is base exception for all API errors. It tries to parse content.detail
    and put it to message.
    """

    def __init__(self, message):
        try:
            self.message = json.loads(message)["detail"]
        except (KeyError, TypeError, ValueError):
            self.message = message
        super().__init__(self.message)



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="BarChart")


@_attrs_define
class BarChart:
    """A class to represent a basic bar chart.

    labels: List[str] the x axis labels
    values: List[int | float] the counts for each bar

        Attributes:
            labels (list[str]):
            values (list[Union[float, int]]):
    """

    labels: list[str]
    values: list[Union[float, int]]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        labels = self.labels

        values = []
        for values_item_data in self.values:
            values_item: Union[float, int]
            values_item = values_item_data
            values.append(values_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"labels": labels, "values": values})

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        labels = cast(list[str], d.pop("labels"))

        values = []
        _values = d.pop("values")
        for values_item_data in _values:

            def _parse_values_item(data: object) -> Union[float, int]:
                return cast(Union[float, int], data)

            values_item = _parse_values_item(values_item_data)

            values.append(values_item)

        bar_chart = cls(labels=labels, values=values)

        bar_chart.additional_properties = d
        return bar_chart

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class CustomizedScorerName(str, Enum):
    VALUE_0 = "_customized_chunk_attribution_utilization_gpt"
    VALUE_1 = "_customized_completeness_gpt"
    VALUE_2 = "_customized_groundedness"
    VALUE_3 = "_customized_factuality"
    VALUE_4 = "_customized_instruction_adherence"
    VALUE_5 = "_customized_tool_error_rate"
    VALUE_6 = "_customized_tool_selection_quality"
    VALUE_7 = "_customized_agentic_workflow_success"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.chain_poll_template import ChainPollTemplate
    from ..models.generated_scorer_configuration import GeneratedScorerConfiguration


T = TypeVar("T", bound="GeneratedScorerValidationRequest")


@_attrs_define
class GeneratedScorerValidationRequest:
    """
    Attributes:
        chain_poll_template (ChainPollTemplate): Template for a chainpoll metric prompt,
            containing all the info necessary to send a chainpoll prompt.
        query (str):
        response (str):
        scorer_configuration (GeneratedScorerConfiguration):
    """

    chain_poll_template: "ChainPollTemplate"
    query: str
    response: str
    scorer_configuration: "GeneratedScorerConfiguration"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        chain_poll_template = self.chain_poll_template.to_dict()

        query = self.query

        response = self.response

        scorer_configuration = self.scorer_configuration.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "chain_poll_template": chain_poll_template,
                "query": query,
                "response": response,
                "scorer_configuration": scorer_configuration,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.chain_poll_template import ChainPollTemplate
        from ..models.generated_scorer_configuration import GeneratedScorerConfiguration

        d = src_dict.copy()
        chain_poll_template = ChainPollTemplate.from_dict(d.pop("chain_poll_template"))

        query = d.pop("query")

        response = d.pop("response")

        scorer_configuration = GeneratedScorerConfiguration.from_dict(d.pop("scorer_configuration"))

        generated_scorer_validation_request = cls(
            chain_poll_template=chain_poll_template,
            query=query,
            response=response,
            scorer_configuration=scorer_configuration,
        )

        generated_scorer_validation_request.additional_properties = d
        return generated_scorer_validation_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Optional

from galileo.api_client import GalileoApiClient


class BaseClientModel:
    """
    A base class for all client data models.
    Used by the model classes to lazy load the API client.

    Args:
        client (Optional[GalileoApiClient], optional): The client to use. Defaults to None.
    """

    client: Optional[GalileoApiClient] = None

    def __init__(self, client: Optional[GalileoApiClient] = None) -> None:
        if client is not None:
            self.client = client
        else:
            self.client = GalileoApiClient()



from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="HasEmbeddingsResponse")


@_attrs_define
class HasEmbeddingsResponse:
    """
    Attributes:
        has_data_embs (Union[Unset, bool]):  Default: False.
        has_model_embs (Union[Unset, bool]):  Default: False.
    """

    has_data_embs: Union[Unset, bool] = False
    has_model_embs: Union[Unset, bool] = False
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        has_data_embs = self.has_data_embs

        has_model_embs = self.has_model_embs

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if has_data_embs is not UNSET:
            field_dict["has_data_embs"] = has_data_embs
        if has_model_embs is not UNSET:
            field_dict["has_model_embs"] = has_model_embs

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        has_data_embs = d.pop("has_data_embs", UNSET)

        has_model_embs = d.pop("has_model_embs", UNSET)

        has_embeddings_response = cls(has_data_embs=has_data_embs, has_model_embs=has_model_embs)

        has_embeddings_response.additional_properties = d
        return has_embeddings_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.user_role import UserRole
from ..types import UNSET, Unset

T = TypeVar("T", bound="InviteUsersRequest")


@_attrs_define
class InviteUsersRequest:
    """
    Attributes:
        emails (list[str]):
        group_ids (Union[Unset, list[str]]):
        role (Union[Unset, UserRole]):
        send_email (Union[Unset, bool]):  Default: True.
    """

    emails: list[str]
    group_ids: Union[Unset, list[str]] = UNSET
    role: Union[Unset, UserRole] = UNSET
    send_email: Union[Unset, bool] = True
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        emails = self.emails

        group_ids: Union[Unset, list[str]] = UNSET
        if not isinstance(self.group_ids, Unset):
            group_ids = self.group_ids

        role: Union[Unset, str] = UNSET
        if not isinstance(self.role, Unset):
            role = self.role.value

        send_email = self.send_email

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"emails": emails})
        if group_ids is not UNSET:
            field_dict["group_ids"] = group_ids
        if role is not UNSET:
            field_dict["role"] = role
        if send_email is not UNSET:
            field_dict["send_email"] = send_email

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        emails = cast(list[str], d.pop("emails"))

        group_ids = cast(list[str], d.pop("group_ids", UNSET))

        _role = d.pop("role", UNSET)
        role: Union[Unset, UserRole]
        if isinstance(_role, Unset):
            role = UNSET
        else:
            role = UserRole(_role)

        send_email = d.pop("send_email", UNSET)

        invite_users_request = cls(emails=emails, group_ids=group_ids, role=role, send_email=send_email)

        invite_users_request.additional_properties = d
        return invite_users_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.group_visibility import GroupVisibility
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.group_member_create import GroupMemberCreate


T = TypeVar("T", bound="GroupCreate")


@_attrs_define
class GroupCreate:
    """
    Attributes:
        name (str):
        description (Union[None, Unset, str]):
        users (Union[Unset, list['GroupMemberCreate']]):
        visibility (Union[Unset, GroupVisibility]):
    """

    name: str
    description: Union[None, Unset, str] = UNSET
    users: Union[Unset, list["GroupMemberCreate"]] = UNSET
    visibility: Union[Unset, GroupVisibility] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        description: Union[None, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        users: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.users, Unset):
            users = []
            for users_item_data in self.users:
                users_item = users_item_data.to_dict()
                users.append(users_item)

        visibility: Union[Unset, str] = UNSET
        if not isinstance(self.visibility, Unset):
            visibility = self.visibility.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"name": name})
        if description is not UNSET:
            field_dict["description"] = description
        if users is not UNSET:
            field_dict["users"] = users
        if visibility is not UNSET:
            field_dict["visibility"] = visibility

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.group_member_create import GroupMemberCreate

        d = src_dict.copy()
        name = d.pop("name")

        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))

        users = []
        _users = d.pop("users", UNSET)
        for users_item_data in _users or []:
            users_item = GroupMemberCreate.from_dict(users_item_data)

            users.append(users_item)

        _visibility = d.pop("visibility", UNSET)
        visibility: Union[Unset, GroupVisibility]
        if isinstance(_visibility, Unset):
            visibility = UNSET
        else:
            visibility = GroupVisibility(_visibility)

        group_create = cls(name=name, description=description, users=users, visibility=visibility)

        group_create.additional_properties = d
        return group_create

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class WebhookStatus(str, Enum):
    ACTIVE = "active"
    FAILED = "failed"
    PAUSED = "paused"
    UNTESTED = "untested"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.get_prompt_run_columns_response import GetPromptRunColumnsResponse
from ...models.http_validation_error import HTTPValidationError
from ...types import UNSET, Response, Unset


def _get_kwargs(
    project_id: str,
    *,
    sort_by: Union[Unset, str] = "id",
    sort_ascending: Union[Unset, bool] = True,
    starting_token: Union[Unset, int] = 0,
    limit: Union[Unset, int] = 100,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    params["sort_by"] = sort_by

    params["sort_ascending"] = sort_ascending

    params["starting_token"] = starting_token

    params["limit"] = limit

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": f"/projects/{project_id}/prompts/runs/columnar",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[GetPromptRunColumnsResponse, HTTPValidationError]]:
    if response.status_code == 200:
        response_200 = GetPromptRunColumnsResponse.from_dict(response.json())

        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[GetPromptRunColumnsResponse, HTTPValidationError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    project_id: str,
    *,
    client: AuthenticatedClient,
    sort_by: Union[Unset, str] = "id",
    sort_ascending: Union[Unset, bool] = True,
    starting_token: Union[Unset, int] = 0,
    limit: Union[Unset, int] = 100,
) -> Response[Union[GetPromptRunColumnsResponse, HTTPValidationError]]:
    """Get Project Prompt Runs As Columns

     Gets all prompt runs for a project in a columnar format.

    This is different from get_project_prompt_runs because it:
    1. Returns results in a columnar format instead of a key-value pair of arrays.
    2. Includes metadata about each of the columns.
    3. Includes mappings to each column's display names and super column names.

    Args:
        project_id (str):
        sort_by (Union[Unset, str]):  Default: 'id'.
        sort_ascending (Union[Unset, bool]):  Default: True.
        starting_token (Union[Unset, int]):  Default: 0.
        limit (Union[Unset, int]):  Default: 100.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[GetPromptRunColumnsResponse, HTTPValidationError]]
    """

    kwargs = _get_kwargs(
        project_id=project_id,
        sort_by=sort_by,
        sort_ascending=sort_ascending,
        starting_token=starting_token,
        limit=limit,
    )

    response = client.get_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


def sync(
    project_id: str,
    *,
    client: AuthenticatedClient,
    sort_by: Union[Unset, str] = "id",
    sort_ascending: Union[Unset, bool] = True,
    starting_token: Union[Unset, int] = 0,
    limit: Union[Unset, int] = 100,
) -> Optional[Union[GetPromptRunColumnsResponse, HTTPValidationError]]:
    """Get Project Prompt Runs As Columns

     Gets all prompt runs for a project in a columnar format.

    This is different from get_project_prompt_runs because it:
    1. Returns results in a columnar format instead of a key-value pair of arrays.
    2. Includes metadata about each of the columns.
    3. Includes mappings to each column's display names and super column names.

    Args:
        project_id (str):
        sort_by (Union[Unset, str]):  Default: 'id'.
        sort_ascending (Union[Unset, bool]):  Default: True.
        starting_token (Union[Unset, int]):  Default: 0.
        limit (Union[Unset, int]):  Default: 100.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[GetPromptRunColumnsResponse, HTTPValidationError]
    """

    return sync_detailed(
        project_id=project_id,
        client=client,
        sort_by=sort_by,
        sort_ascending=sort_ascending,
        starting_token=starting_token,
        limit=limit,
    ).parsed


async def asyncio_detailed(
    project_id: str,
    *,
    client: AuthenticatedClient,
    sort_by: Union[Unset, str] = "id",
    sort_ascending: Union[Unset, bool] = True,
    starting_token: Union[Unset, int] = 0,
    limit: Union[Unset, int] = 100,
) -> Response[Union[GetPromptRunColumnsResponse, HTTPValidationError]]:
    """Get Project Prompt Runs As Columns

     Gets all prompt runs for a project in a columnar format.

    This is different from get_project_prompt_runs because it:
    1. Returns results in a columnar format instead of a key-value pair of arrays.
    2. Includes metadata about each of the columns.
    3. Includes mappings to each column's display names and super column names.

    Args:
        project_id (str):
        sort_by (Union[Unset, str]):  Default: 'id'.
        sort_ascending (Union[Unset, bool]):  Default: True.
        starting_token (Union[Unset, int]):  Default: 0.
        limit (Union[Unset, int]):  Default: 100.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[GetPromptRunColumnsResponse, HTTPValidationError]]
    """

    kwargs = _get_kwargs(
        project_id=project_id,
        sort_by=sort_by,
        sort_ascending=sort_ascending,
        starting_token=starting_token,
        limit=limit,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    project_id: str,
    *,
    client: AuthenticatedClient,
    sort_by: Union[Unset, str] = "id",
    sort_ascending: Union[Unset, bool] = True,
    starting_token: Union[Unset, int] = 0,
    limit: Union[Unset, int] = 100,
) -> Optional[Union[GetPromptRunColumnsResponse, HTTPValidationError]]:
    """Get Project Prompt Runs As Columns

     Gets all prompt runs for a project in a columnar format.

    This is different from get_project_prompt_runs because it:
    1. Returns results in a columnar format instead of a key-value pair of arrays.
    2. Includes metadata about each of the columns.
    3. Includes mappings to each column's display names and super column names.

    Args:
        project_id (str):
        sort_by (Union[Unset, str]):  Default: 'id'.
        sort_ascending (Union[Unset, bool]):  Default: True.
        starting_token (Union[Unset, int]):  Default: 0.
        limit (Union[Unset, int]):  Default: 100.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[GetPromptRunColumnsResponse, HTTPValidationError]
    """

    return (
        await asyncio_detailed(
            project_id=project_id,
            client=client,
            sort_by=sort_by,
            sort_ascending=sort_ascending,
            starting_token=starting_token,
            limit=limit,
        )
    ).parsed



from typing import Any, Literal, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="RecomputeSettingsObserveBase")


@_attrs_define
class RecomputeSettingsObserveBase:
    """
    Attributes:
        filters (list[Any]):
        mode (Union[Literal['observe_filters'], Unset]):  Default: 'observe_filters'.
    """

    filters: list[Any]
    mode: Union[Literal["observe_filters"], Unset] = "observe_filters"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        filters = self.filters

        mode = self.mode

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"filters": filters})
        if mode is not UNSET:
            field_dict["mode"] = mode

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        filters = cast(list[Any], d.pop("filters"))

        mode = cast(Union[Literal["observe_filters"], Unset], d.pop("mode", UNSET))
        if mode != "observe_filters" and not isinstance(mode, Unset):
            raise ValueError(f"mode must match const 'observe_filters', got '{mode}'")

        recompute_settings_observe_base = cls(filters=filters, mode=mode)

        recompute_settings_observe_base.additional_properties = d
        return recompute_settings_observe_base

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.scorer_config import ScorerConfig


T = TypeVar("T", bound="RunScorerSettingsRequest")


@_attrs_define
class RunScorerSettingsRequest:
    """
    Attributes:
        run_id (str): ID of the run.
        scorers (Union[Unset, list['ScorerConfig']]): List of Galileo scorers to enable.
    """

    run_id: str
    scorers: Union[Unset, list["ScorerConfig"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        run_id = self.run_id

        scorers: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.scorers, Unset):
            scorers = []
            for scorers_item_data in self.scorers:
                scorers_item = scorers_item_data.to_dict()
                scorers.append(scorers_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"run_id": run_id})
        if scorers is not UNSET:
            field_dict["scorers"] = scorers

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.scorer_config import ScorerConfig

        d = src_dict.copy()
        run_id = d.pop("run_id")

        scorers = []
        _scorers = d.pop("scorers", UNSET)
        for scorers_item_data in _scorers or []:
            scorers_item = ScorerConfig.from_dict(scorers_item_data)

            scorers.append(scorers_item)

        run_scorer_settings_request = cls(run_id=run_id, scorers=scorers)

        run_scorer_settings_request.additional_properties = d
        return run_scorer_settings_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.node_type import NodeType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.chain_poll_template import ChainPollTemplate
    from ..models.generated_scorer_configuration import GeneratedScorerConfiguration


T = TypeVar("T", bound="UpdateGeneratedScorerRequest")


@_attrs_define
class UpdateGeneratedScorerRequest:
    """
    Attributes:
        chain_poll_template (Union['ChainPollTemplate', None, Unset]):
        instructions (Union[None, Unset, str]):
        name (Union[None, Unset, str]):
        scoreable_node_types (Union[None, Unset, list[NodeType]]):
        scorer_configuration (Union['GeneratedScorerConfiguration', None, Unset]):
    """

    chain_poll_template: Union["ChainPollTemplate", None, Unset] = UNSET
    instructions: Union[None, Unset, str] = UNSET
    name: Union[None, Unset, str] = UNSET
    scoreable_node_types: Union[None, Unset, list[NodeType]] = UNSET
    scorer_configuration: Union["GeneratedScorerConfiguration", None, Unset] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.chain_poll_template import ChainPollTemplate
        from ..models.generated_scorer_configuration import GeneratedScorerConfiguration

        chain_poll_template: Union[None, Unset, dict[str, Any]]
        if isinstance(self.chain_poll_template, Unset):
            chain_poll_template = UNSET
        elif isinstance(self.chain_poll_template, ChainPollTemplate):
            chain_poll_template = self.chain_poll_template.to_dict()
        else:
            chain_poll_template = self.chain_poll_template

        instructions: Union[None, Unset, str]
        if isinstance(self.instructions, Unset):
            instructions = UNSET
        else:
            instructions = self.instructions

        name: Union[None, Unset, str]
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        scoreable_node_types: Union[None, Unset, list[str]]
        if isinstance(self.scoreable_node_types, Unset):
            scoreable_node_types = UNSET
        elif isinstance(self.scoreable_node_types, list):
            scoreable_node_types = []
            for scoreable_node_types_type_0_item_data in self.scoreable_node_types:
                scoreable_node_types_type_0_item = scoreable_node_types_type_0_item_data.value
                scoreable_node_types.append(scoreable_node_types_type_0_item)

        else:
            scoreable_node_types = self.scoreable_node_types

        scorer_configuration: Union[None, Unset, dict[str, Any]]
        if isinstance(self.scorer_configuration, Unset):
            scorer_configuration = UNSET
        elif isinstance(self.scorer_configuration, GeneratedScorerConfiguration):
            scorer_configuration = self.scorer_configuration.to_dict()
        else:
            scorer_configuration = self.scorer_configuration

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if chain_poll_template is not UNSET:
            field_dict["chain_poll_template"] = chain_poll_template
        if instructions is not UNSET:
            field_dict["instructions"] = instructions
        if name is not UNSET:
            field_dict["name"] = name
        if scoreable_node_types is not UNSET:
            field_dict["scoreable_node_types"] = scoreable_node_types
        if scorer_configuration is not UNSET:
            field_dict["scorer_configuration"] = scorer_configuration

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.chain_poll_template import ChainPollTemplate
        from ..models.generated_scorer_configuration import GeneratedScorerConfiguration

        d = src_dict.copy()

        def _parse_chain_poll_template(data: object) -> Union["ChainPollTemplate", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                chain_poll_template_type_0 = ChainPollTemplate.from_dict(data)

                return chain_poll_template_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ChainPollTemplate", None, Unset], data)

        chain_poll_template = _parse_chain_poll_template(d.pop("chain_poll_template", UNSET))

        def _parse_instructions(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        instructions = _parse_instructions(d.pop("instructions", UNSET))

        def _parse_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        name = _parse_name(d.pop("name", UNSET))

        def _parse_scoreable_node_types(data: object) -> Union[None, Unset, list[NodeType]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                scoreable_node_types_type_0 = []
                _scoreable_node_types_type_0 = data
                for scoreable_node_types_type_0_item_data in _scoreable_node_types_type_0:
                    scoreable_node_types_type_0_item = NodeType(scoreable_node_types_type_0_item_data)

                    scoreable_node_types_type_0.append(scoreable_node_types_type_0_item)

                return scoreable_node_types_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[NodeType]], data)

        scoreable_node_types = _parse_scoreable_node_types(d.pop("scoreable_node_types", UNSET))

        def _parse_scorer_configuration(data: object) -> Union["GeneratedScorerConfiguration", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                scorer_configuration_type_0 = GeneratedScorerConfiguration.from_dict(data)

                return scorer_configuration_type_0
            except:  # noqa: E722
                pass
            return cast(Union["GeneratedScorerConfiguration", None, Unset], data)

        scorer_configuration = _parse_scorer_configuration(d.pop("scorer_configuration", UNSET))

        update_generated_scorer_request = cls(
            chain_poll_template=chain_poll_template,
            instructions=instructions,
            name=name,
            scoreable_node_types=scoreable_node_types,
            scorer_configuration=scorer_configuration,
        )

        update_generated_scorer_request.additional_properties = d
        return update_generated_scorer_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="DatasetRow")


@_attrs_define
class DatasetRow:
    """
    Attributes:
        index (int):
        values (list[Union[None, float, int, str]]):
    """

    index: int
    values: list[Union[None, float, int, str]]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        index = self.index

        values = []
        for values_item_data in self.values:
            values_item: Union[None, float, int, str]
            values_item = values_item_data
            values.append(values_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"index": index, "values": values})

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        index = d.pop("index")

        values = []
        _values = d.pop("values")
        for values_item_data in _values:

            def _parse_values_item(data: object) -> Union[None, float, int, str]:
                if data is None:
                    return data
                return cast(Union[None, float, int, str], data)

            values_item = _parse_values_item(values_item_data)

            values.append(values_item)

        dataset_row = cls(index=index, values=values)

        dataset_row.additional_properties = d
        return dataset_row

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Metrics")


@_attrs_define
class Metrics:
    """
    Attributes:
        bleu (Union[None, Unset, float]):
        context_relevance (Union[None, Unset, float]):
        factuality (Union[None, Unset, float]):
        groundedness (Union[None, Unset, float]):
        hallucination (Union[None, Unset, float]):
        instruction_adherence (Union[None, Unset, float]):
        latency (Union[None, Unset, float]):
        pii (Union[None, Unset, list[str]]):
        rouge (Union[None, Unset, float]):
        toxicity (Union[None, Unset, float]):
    """

    bleu: Union[None, Unset, float] = UNSET
    context_relevance: Union[None, Unset, float] = UNSET
    factuality: Union[None, Unset, float] = UNSET
    groundedness: Union[None, Unset, float] = UNSET
    hallucination: Union[None, Unset, float] = UNSET
    instruction_adherence: Union[None, Unset, float] = UNSET
    latency: Union[None, Unset, float] = UNSET
    pii: Union[None, Unset, list[str]] = UNSET
    rouge: Union[None, Unset, float] = UNSET
    toxicity: Union[None, Unset, float] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        bleu: Union[None, Unset, float]
        if isinstance(self.bleu, Unset):
            bleu = UNSET
        else:
            bleu = self.bleu

        context_relevance: Union[None, Unset, float]
        if isinstance(self.context_relevance, Unset):
            context_relevance = UNSET
        else:
            context_relevance = self.context_relevance

        factuality: Union[None, Unset, float]
        if isinstance(self.factuality, Unset):
            factuality = UNSET
        else:
            factuality = self.factuality

        groundedness: Union[None, Unset, float]
        if isinstance(self.groundedness, Unset):
            groundedness = UNSET
        else:
            groundedness = self.groundedness

        hallucination: Union[None, Unset, float]
        if isinstance(self.hallucination, Unset):
            hallucination = UNSET
        else:
            hallucination = self.hallucination

        instruction_adherence: Union[None, Unset, float]
        if isinstance(self.instruction_adherence, Unset):
            instruction_adherence = UNSET
        else:
            instruction_adherence = self.instruction_adherence

        latency: Union[None, Unset, float]
        if isinstance(self.latency, Unset):
            latency = UNSET
        else:
            latency = self.latency

        pii: Union[None, Unset, list[str]]
        if isinstance(self.pii, Unset):
            pii = UNSET
        elif isinstance(self.pii, list):
            pii = self.pii

        else:
            pii = self.pii

        rouge: Union[None, Unset, float]
        if isinstance(self.rouge, Unset):
            rouge = UNSET
        else:
            rouge = self.rouge

        toxicity: Union[None, Unset, float]
        if isinstance(self.toxicity, Unset):
            toxicity = UNSET
        else:
            toxicity = self.toxicity

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if bleu is not UNSET:
            field_dict["bleu"] = bleu
        if context_relevance is not UNSET:
            field_dict["context_relevance"] = context_relevance
        if factuality is not UNSET:
            field_dict["factuality"] = factuality
        if groundedness is not UNSET:
            field_dict["groundedness"] = groundedness
        if hallucination is not UNSET:
            field_dict["hallucination"] = hallucination
        if instruction_adherence is not UNSET:
            field_dict["instruction_adherence"] = instruction_adherence
        if latency is not UNSET:
            field_dict["latency"] = latency
        if pii is not UNSET:
            field_dict["pii"] = pii
        if rouge is not UNSET:
            field_dict["rouge"] = rouge
        if toxicity is not UNSET:
            field_dict["toxicity"] = toxicity

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_bleu(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        bleu = _parse_bleu(d.pop("bleu", UNSET))

        def _parse_context_relevance(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        context_relevance = _parse_context_relevance(d.pop("context_relevance", UNSET))

        def _parse_factuality(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        factuality = _parse_factuality(d.pop("factuality", UNSET))

        def _parse_groundedness(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        groundedness = _parse_groundedness(d.pop("groundedness", UNSET))

        def _parse_hallucination(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        hallucination = _parse_hallucination(d.pop("hallucination", UNSET))

        def _parse_instruction_adherence(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        instruction_adherence = _parse_instruction_adherence(d.pop("instruction_adherence", UNSET))

        def _parse_latency(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        latency = _parse_latency(d.pop("latency", UNSET))

        def _parse_pii(data: object) -> Union[None, Unset, list[str]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                pii_type_0 = cast(list[str], data)

                return pii_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[str]], data)

        pii = _parse_pii(d.pop("pii", UNSET))

        def _parse_rouge(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        rouge = _parse_rouge(d.pop("rouge", UNSET))

        def _parse_toxicity(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        toxicity = _parse_toxicity(d.pop("toxicity", UNSET))

        metrics = cls(
            bleu=bleu,
            context_relevance=context_relevance,
            factuality=factuality,
            groundedness=groundedness,
            hallucination=hallucination,
            instruction_adherence=instruction_adherence,
            latency=latency,
            pii=pii,
            rouge=rouge,
            toxicity=toxicity,
        )

        metrics.additional_properties = d
        return metrics

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.webhook_status import WebhookStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="Webhook")


@_attrs_define
class Webhook:
    """
    Attributes:
        url (str):
        notes (Union[Unset, str]):  Default: ''.
        status (Union[Unset, WebhookStatus]):
    """

    url: str
    notes: Union[Unset, str] = ""
    status: Union[Unset, WebhookStatus] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        url = self.url

        notes = self.notes

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"url": url})
        if notes is not UNSET:
            field_dict["notes"] = notes
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        url = d.pop("url")

        notes = d.pop("notes", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, WebhookStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = WebhookStatus(_status)

        webhook = cls(url=url, notes=notes, status=status)

        webhook.additional_properties = d
        return webhook

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class SemSegErrorType(str, Enum):
    BACKGROUND = "background"
    CLASSIFICATION = "classification"
    CLASS_CONFUSION = "class_confusion"
    MISSED = "missed"
    NONE = "None"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.dataset_version_index_sort import DatasetVersionIndexSort


T = TypeVar("T", bound="ListDatasetVersionParams")


@_attrs_define
class ListDatasetVersionParams:
    """
    Attributes:
        sort (Union['DatasetVersionIndexSort', None, Unset]):
    """

    sort: Union["DatasetVersionIndexSort", None, Unset] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.dataset_version_index_sort import DatasetVersionIndexSort

        sort: Union[None, Unset, dict[str, Any]]
        if isinstance(self.sort, Unset):
            sort = UNSET
        elif isinstance(self.sort, DatasetVersionIndexSort):
            sort = self.sort.to_dict()
        else:
            sort = self.sort

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if sort is not UNSET:
            field_dict["sort"] = sort

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.dataset_version_index_sort import DatasetVersionIndexSort

        d = src_dict.copy()

        def _parse_sort(data: object) -> Union["DatasetVersionIndexSort", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                sort_type_0 = DatasetVersionIndexSort.from_dict(data)

                return sort_type_0
            except:  # noqa: E722
                pass
            return cast(Union["DatasetVersionIndexSort", None, Unset], data)

        sort = _parse_sort(d.pop("sort", UNSET))

        list_dataset_version_params = cls(sort=sort)

        list_dataset_version_params.additional_properties = d
        return list_dataset_version_params

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.metric_critique_content import MetricCritiqueContent
    from ..models.metric_inputs import MetricInputs


T = TypeVar("T", bound="MetricCritique")


@_attrs_define
class MetricCritique:
    """
    Attributes:
        critique_info (MetricCritiqueContent):
        id (str):
        is_computed (bool):
        metric (str):
        metric_inputs (MetricInputs): Metric inputs for a prompt template.
            Include an optional inputs_config to help formatting with a prompt template.
        project_id (str):
        revised_explanation (Union[None, str]):
        row_id (Union[UUID, int]):
        run_id (Union[None, str]):
    """

    critique_info: "MetricCritiqueContent"
    id: str
    is_computed: bool
    metric: str
    metric_inputs: "MetricInputs"
    project_id: str
    revised_explanation: Union[None, str]
    row_id: Union[UUID, int]
    run_id: Union[None, str]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        critique_info = self.critique_info.to_dict()

        id = self.id

        is_computed = self.is_computed

        metric = self.metric

        metric_inputs = self.metric_inputs.to_dict()

        project_id = self.project_id

        revised_explanation: Union[None, str]
        revised_explanation = self.revised_explanation

        row_id: Union[int, str]
        if isinstance(self.row_id, UUID):
            row_id = str(self.row_id)
        else:
            row_id = self.row_id

        run_id: Union[None, str]
        run_id = self.run_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "critique_info": critique_info,
                "id": id,
                "is_computed": is_computed,
                "metric": metric,
                "metric_inputs": metric_inputs,
                "project_id": project_id,
                "revised_explanation": revised_explanation,
                "row_id": row_id,
                "run_id": run_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.metric_critique_content import MetricCritiqueContent
        from ..models.metric_inputs import MetricInputs

        d = src_dict.copy()
        critique_info = MetricCritiqueContent.from_dict(d.pop("critique_info"))

        id = d.pop("id")

        is_computed = d.pop("is_computed")

        metric = d.pop("metric")

        metric_inputs = MetricInputs.from_dict(d.pop("metric_inputs"))

        project_id = d.pop("project_id")

        def _parse_revised_explanation(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        revised_explanation = _parse_revised_explanation(d.pop("revised_explanation"))

        def _parse_row_id(data: object) -> Union[UUID, int]:
            try:
                if not isinstance(data, str):
                    raise TypeError()
                row_id_type_0 = UUID(data)

                return row_id_type_0
            except:  # noqa: E722
                pass
            return cast(Union[UUID, int], data)

        row_id = _parse_row_id(d.pop("row_id"))

        def _parse_run_id(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        run_id = _parse_run_id(d.pop("run_id"))

        metric_critique = cls(
            critique_info=critique_info,
            id=id,
            is_computed=is_computed,
            metric=metric,
            metric_inputs=metric_inputs,
            project_id=project_id,
            revised_explanation=revised_explanation,
            row_id=row_id,
            run_id=run_id,
        )

        metric_critique.additional_properties = d
        return metric_critique

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="GeneratedScorerConfiguration")


@_attrs_define
class GeneratedScorerConfiguration:
    """
    Attributes:
        model_alias (Union[None, Unset, str]):  Default: 'GPT-4o mini'.
        num_judges (Union[None, Unset, int]):  Default: 3.
    """

    model_alias: Union[None, Unset, str] = "GPT-4o mini"
    num_judges: Union[None, Unset, int] = 3
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        model_alias: Union[None, Unset, str]
        if isinstance(self.model_alias, Unset):
            model_alias = UNSET
        else:
            model_alias = self.model_alias

        num_judges: Union[None, Unset, int]
        if isinstance(self.num_judges, Unset):
            num_judges = UNSET
        else:
            num_judges = self.num_judges

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if model_alias is not UNSET:
            field_dict["model_alias"] = model_alias
        if num_judges is not UNSET:
            field_dict["num_judges"] = num_judges

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_model_alias(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        model_alias = _parse_model_alias(d.pop("model_alias", UNSET))

        def _parse_num_judges(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        num_judges = _parse_num_judges(d.pop("num_judges", UNSET))

        generated_scorer_configuration = cls(model_alias=model_alias, num_judges=num_judges)

        generated_scorer_configuration.additional_properties = d
        return generated_scorer_configuration

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.auth_method import AuthMethod
from ..models.user_role import UserRole
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.permission import Permission


T = TypeVar("T", bound="CreateUserResponse")


@_attrs_define
class CreateUserResponse:
    """
    Attributes:
        created_at (datetime.datetime):
        email (str):
        id (str):
        organization_id (str):
        organization_name (str):
        updated_at (datetime.datetime):
        auth_method (Union[Unset, AuthMethod]):
        email_is_verified (Union[None, Unset, bool]):
        first_name (Union[None, Unset, str]):  Default: ''.
        last_name (Union[None, Unset, str]):  Default: ''.
        permissions (Union[Unset, list['Permission']]):
        role (Union[Unset, UserRole]):
    """

    created_at: datetime.datetime
    email: str
    id: str
    organization_id: str
    organization_name: str
    updated_at: datetime.datetime
    auth_method: Union[Unset, AuthMethod] = UNSET
    email_is_verified: Union[None, Unset, bool] = UNSET
    first_name: Union[None, Unset, str] = ""
    last_name: Union[None, Unset, str] = ""
    permissions: Union[Unset, list["Permission"]] = UNSET
    role: Union[Unset, UserRole] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        created_at = self.created_at.isoformat()

        email = self.email

        id = self.id

        organization_id = self.organization_id

        organization_name = self.organization_name

        updated_at = self.updated_at.isoformat()

        auth_method: Union[Unset, str] = UNSET
        if not isinstance(self.auth_method, Unset):
            auth_method = self.auth_method.value

        email_is_verified: Union[None, Unset, bool]
        if isinstance(self.email_is_verified, Unset):
            email_is_verified = UNSET
        else:
            email_is_verified = self.email_is_verified

        first_name: Union[None, Unset, str]
        if isinstance(self.first_name, Unset):
            first_name = UNSET
        else:
            first_name = self.first_name

        last_name: Union[None, Unset, str]
        if isinstance(self.last_name, Unset):
            last_name = UNSET
        else:
            last_name = self.last_name

        permissions: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.permissions, Unset):
            permissions = []
            for permissions_item_data in self.permissions:
                permissions_item = permissions_item_data.to_dict()
                permissions.append(permissions_item)

        role: Union[Unset, str] = UNSET
        if not isinstance(self.role, Unset):
            role = self.role.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "created_at": created_at,
                "email": email,
                "id": id,
                "organization_id": organization_id,
                "organization_name": organization_name,
                "updated_at": updated_at,
            }
        )
        if auth_method is not UNSET:
            field_dict["auth_method"] = auth_method
        if email_is_verified is not UNSET:
            field_dict["email_is_verified"] = email_is_verified
        if first_name is not UNSET:
            field_dict["first_name"] = first_name
        if last_name is not UNSET:
            field_dict["last_name"] = last_name
        if permissions is not UNSET:
            field_dict["permissions"] = permissions
        if role is not UNSET:
            field_dict["role"] = role

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.permission import Permission

        d = src_dict.copy()
        created_at = isoparse(d.pop("created_at"))

        email = d.pop("email")

        id = d.pop("id")

        organization_id = d.pop("organization_id")

        organization_name = d.pop("organization_name")

        updated_at = isoparse(d.pop("updated_at"))

        _auth_method = d.pop("auth_method", UNSET)
        auth_method: Union[Unset, AuthMethod]
        if isinstance(_auth_method, Unset):
            auth_method = UNSET
        else:
            auth_method = AuthMethod(_auth_method)

        def _parse_email_is_verified(data: object) -> Union[None, Unset, bool]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, bool], data)

        email_is_verified = _parse_email_is_verified(d.pop("email_is_verified", UNSET))

        def _parse_first_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        first_name = _parse_first_name(d.pop("first_name", UNSET))

        def _parse_last_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        last_name = _parse_last_name(d.pop("last_name", UNSET))

        permissions = []
        _permissions = d.pop("permissions", UNSET)
        for permissions_item_data in _permissions or []:
            permissions_item = Permission.from_dict(permissions_item_data)

            permissions.append(permissions_item)

        _role = d.pop("role", UNSET)
        role: Union[Unset, UserRole]
        if isinstance(_role, Unset):
            role = UNSET
        else:
            role = UserRole(_role)

        create_user_response = cls(
            created_at=created_at,
            email=email,
            id=id,
            organization_id=organization_id,
            organization_name=organization_name,
            updated_at=updated_at,
            auth_method=auth_method,
            email_is_verified=email_is_verified,
            first_name=first_name,
            last_name=last_name,
            permissions=permissions,
            role=role,
        )

        create_user_response.additional_properties = d
        return create_user_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="UsagePlan")


@_attrs_define
class UsagePlan:
    """
    Attributes:
        max_data_visibility_days (int):
        max_organizations_per_user (int):
        max_payload_size_bytes (int):
        max_traces_per_month (int):
        max_users_per_organization (int):
    """

    max_data_visibility_days: int
    max_organizations_per_user: int
    max_payload_size_bytes: int
    max_traces_per_month: int
    max_users_per_organization: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        max_data_visibility_days = self.max_data_visibility_days

        max_organizations_per_user = self.max_organizations_per_user

        max_payload_size_bytes = self.max_payload_size_bytes

        max_traces_per_month = self.max_traces_per_month

        max_users_per_organization = self.max_users_per_organization

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "max_data_visibility_days": max_data_visibility_days,
                "max_organizations_per_user": max_organizations_per_user,
                "max_payload_size_bytes": max_payload_size_bytes,
                "max_traces_per_month": max_traces_per_month,
                "max_users_per_organization": max_users_per_organization,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        max_data_visibility_days = d.pop("max_data_visibility_days")

        max_organizations_per_user = d.pop("max_organizations_per_user")

        max_payload_size_bytes = d.pop("max_payload_size_bytes")

        max_traces_per_month = d.pop("max_traces_per_month")

        max_users_per_organization = d.pop("max_users_per_organization")

        usage_plan = cls(
            max_data_visibility_days=max_data_visibility_days,
            max_organizations_per_user=max_organizations_per_user,
            max_payload_size_bytes=max_payload_size_bytes,
            max_traces_per_month=max_traces_per_month,
            max_users_per_organization=max_users_per_organization,
        )

        usage_plan.additional_properties = d
        return usage_plan

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.node_type import NodeType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.chain_row_inputs_type_0 import ChainRowInputsType0
    from ..models.chain_row_params import ChainRowParams


T = TypeVar("T", bound="ChainRow")


@_attrs_define
class ChainRow:
    """
    Attributes:
        chain_root_id (UUID):
        node_id (UUID):
        node_type (NodeType):
        chain_id (Union[None, UUID, Unset]):
        creation_timestamp (Union[Unset, int]):
        expected_output (Union[None, Unset, str]):
        finish_reason (Union[Unset, str]):  Default: ''.
        has_children (Union[Unset, bool]):  Default: False.
        inputs (Union['ChainRowInputsType0', None, Unset]):
        latency (Union[None, Unset, int]):
        node_input (Union[Unset, str]):  Default: ''.
        node_name (Union[None, Unset, str]):
        node_output (Union[Unset, str]):  Default: ''.
        params (Union[Unset, ChainRowParams]):
        prompt (Union[None, Unset, str]):
        query_input_tokens (Union[Unset, int]):  Default: 0.
        query_output_tokens (Union[Unset, int]):  Default: 0.
        query_total_tokens (Union[Unset, int]):  Default: 0.
        response (Union[None, Unset, str]):
        step (Union[Unset, int]):  Default: 0.
        tools (Union[None, Unset, str]):
    """

    chain_root_id: UUID
    node_id: UUID
    node_type: NodeType
    chain_id: Union[None, UUID, Unset] = UNSET
    creation_timestamp: Union[Unset, int] = UNSET
    expected_output: Union[None, Unset, str] = UNSET
    finish_reason: Union[Unset, str] = ""
    has_children: Union[Unset, bool] = False
    inputs: Union["ChainRowInputsType0", None, Unset] = UNSET
    latency: Union[None, Unset, int] = UNSET
    node_input: Union[Unset, str] = ""
    node_name: Union[None, Unset, str] = UNSET
    node_output: Union[Unset, str] = ""
    params: Union[Unset, "ChainRowParams"] = UNSET
    prompt: Union[None, Unset, str] = UNSET
    query_input_tokens: Union[Unset, int] = 0
    query_output_tokens: Union[Unset, int] = 0
    query_total_tokens: Union[Unset, int] = 0
    response: Union[None, Unset, str] = UNSET
    step: Union[Unset, int] = 0
    tools: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.chain_row_inputs_type_0 import ChainRowInputsType0

        chain_root_id = str(self.chain_root_id)

        node_id = str(self.node_id)

        node_type = self.node_type.value

        chain_id: Union[None, Unset, str]
        if isinstance(self.chain_id, Unset):
            chain_id = UNSET
        elif isinstance(self.chain_id, UUID):
            chain_id = str(self.chain_id)
        else:
            chain_id = self.chain_id

        creation_timestamp = self.creation_timestamp

        expected_output: Union[None, Unset, str]
        if isinstance(self.expected_output, Unset):
            expected_output = UNSET
        else:
            expected_output = self.expected_output

        finish_reason = self.finish_reason

        has_children = self.has_children

        inputs: Union[None, Unset, dict[str, Any]]
        if isinstance(self.inputs, Unset):
            inputs = UNSET
        elif isinstance(self.inputs, ChainRowInputsType0):
            inputs = self.inputs.to_dict()
        else:
            inputs = self.inputs

        latency: Union[None, Unset, int]
        if isinstance(self.latency, Unset):
            latency = UNSET
        else:
            latency = self.latency

        node_input = self.node_input

        node_name: Union[None, Unset, str]
        if isinstance(self.node_name, Unset):
            node_name = UNSET
        else:
            node_name = self.node_name

        node_output = self.node_output

        params: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.params, Unset):
            params = self.params.to_dict()

        prompt: Union[None, Unset, str]
        if isinstance(self.prompt, Unset):
            prompt = UNSET
        else:
            prompt = self.prompt

        query_input_tokens = self.query_input_tokens

        query_output_tokens = self.query_output_tokens

        query_total_tokens = self.query_total_tokens

        response: Union[None, Unset, str]
        if isinstance(self.response, Unset):
            response = UNSET
        else:
            response = self.response

        step = self.step

        tools: Union[None, Unset, str]
        if isinstance(self.tools, Unset):
            tools = UNSET
        else:
            tools = self.tools

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"chain_root_id": chain_root_id, "node_id": node_id, "node_type": node_type})
        if chain_id is not UNSET:
            field_dict["chain_id"] = chain_id
        if creation_timestamp is not UNSET:
            field_dict["creation_timestamp"] = creation_timestamp
        if expected_output is not UNSET:
            field_dict["expected_output"] = expected_output
        if finish_reason is not UNSET:
            field_dict["finish_reason"] = finish_reason
        if has_children is not UNSET:
            field_dict["has_children"] = has_children
        if inputs is not UNSET:
            field_dict["inputs"] = inputs
        if latency is not UNSET:
            field_dict["latency"] = latency
        if node_input is not UNSET:
            field_dict["node_input"] = node_input
        if node_name is not UNSET:
            field_dict["node_name"] = node_name
        if node_output is not UNSET:
            field_dict["node_output"] = node_output
        if params is not UNSET:
            field_dict["params"] = params
        if prompt is not UNSET:
            field_dict["prompt"] = prompt
        if query_input_tokens is not UNSET:
            field_dict["query_input_tokens"] = query_input_tokens
        if query_output_tokens is not UNSET:
            field_dict["query_output_tokens"] = query_output_tokens
        if query_total_tokens is not UNSET:
            field_dict["query_total_tokens"] = query_total_tokens
        if response is not UNSET:
            field_dict["response"] = response
        if step is not UNSET:
            field_dict["step"] = step
        if tools is not UNSET:
            field_dict["tools"] = tools

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.chain_row_inputs_type_0 import ChainRowInputsType0
        from ..models.chain_row_params import ChainRowParams

        d = src_dict.copy()
        chain_root_id = UUID(d.pop("chain_root_id"))

        node_id = UUID(d.pop("node_id"))

        node_type = NodeType(d.pop("node_type"))

        def _parse_chain_id(data: object) -> Union[None, UUID, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                chain_id_type_0 = UUID(data)

                return chain_id_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, UUID, Unset], data)

        chain_id = _parse_chain_id(d.pop("chain_id", UNSET))

        creation_timestamp = d.pop("creation_timestamp", UNSET)

        def _parse_expected_output(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        expected_output = _parse_expected_output(d.pop("expected_output", UNSET))

        finish_reason = d.pop("finish_reason", UNSET)

        has_children = d.pop("has_children", UNSET)

        def _parse_inputs(data: object) -> Union["ChainRowInputsType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                inputs_type_0 = ChainRowInputsType0.from_dict(data)

                return inputs_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ChainRowInputsType0", None, Unset], data)

        inputs = _parse_inputs(d.pop("inputs", UNSET))

        def _parse_latency(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        latency = _parse_latency(d.pop("latency", UNSET))

        node_input = d.pop("node_input", UNSET)

        def _parse_node_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        node_name = _parse_node_name(d.pop("node_name", UNSET))

        node_output = d.pop("node_output", UNSET)

        _params = d.pop("params", UNSET)
        params: Union[Unset, ChainRowParams]
        if isinstance(_params, Unset):
            params = UNSET
        else:
            params = ChainRowParams.from_dict(_params)

        def _parse_prompt(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        prompt = _parse_prompt(d.pop("prompt", UNSET))

        query_input_tokens = d.pop("query_input_tokens", UNSET)

        query_output_tokens = d.pop("query_output_tokens", UNSET)

        query_total_tokens = d.pop("query_total_tokens", UNSET)

        def _parse_response(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        response = _parse_response(d.pop("response", UNSET))

        step = d.pop("step", UNSET)

        def _parse_tools(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        tools = _parse_tools(d.pop("tools", UNSET))

        chain_row = cls(
            chain_root_id=chain_root_id,
            node_id=node_id,
            node_type=node_type,
            chain_id=chain_id,
            creation_timestamp=creation_timestamp,
            expected_output=expected_output,
            finish_reason=finish_reason,
            has_children=has_children,
            inputs=inputs,
            latency=latency,
            node_input=node_input,
            node_name=node_name,
            node_output=node_output,
            params=params,
            prompt=prompt,
            query_input_tokens=query_input_tokens,
            query_output_tokens=query_output_tokens,
            query_total_tokens=query_total_tokens,
            response=response,
            step=step,
            tools=tools,
        )

        chain_row.additional_properties = d
        return chain_row

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.log_data_logging_method import LogDataLoggingMethod
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.trace import Trace


T = TypeVar("T", bound="TracesIngestRequest")


@_attrs_define
class TracesIngestRequest:
    """
    Attributes:
        traces (list['Trace']): List of traces to log.
        client_version (Union[None, Unset, str]):
        experiment_id (Union[None, Unset, str]): Experiment id associated with the traces.
        log_stream_id (Union[None, Unset, str]): Log stream id associated with the traces.
        logging_method (Union[Unset, LogDataLoggingMethod]):
    """

    traces: list["Trace"]
    client_version: Union[None, Unset, str] = UNSET
    experiment_id: Union[None, Unset, str] = UNSET
    log_stream_id: Union[None, Unset, str] = UNSET
    logging_method: Union[Unset, LogDataLoggingMethod] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        traces = []
        for traces_item_data in self.traces:
            traces_item = traces_item_data.to_dict()
            traces.append(traces_item)

        client_version: Union[None, Unset, str]
        if isinstance(self.client_version, Unset):
            client_version = UNSET
        else:
            client_version = self.client_version

        experiment_id: Union[None, Unset, str]
        if isinstance(self.experiment_id, Unset):
            experiment_id = UNSET
        else:
            experiment_id = self.experiment_id

        log_stream_id: Union[None, Unset, str]
        if isinstance(self.log_stream_id, Unset):
            log_stream_id = UNSET
        else:
            log_stream_id = self.log_stream_id

        logging_method: Union[Unset, str] = UNSET
        if not isinstance(self.logging_method, Unset):
            logging_method = self.logging_method.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"traces": traces})
        if client_version is not UNSET:
            field_dict["client_version"] = client_version
        if experiment_id is not UNSET:
            field_dict["experiment_id"] = experiment_id
        if log_stream_id is not UNSET:
            field_dict["log_stream_id"] = log_stream_id
        if logging_method is not UNSET:
            field_dict["logging_method"] = logging_method

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.trace import Trace

        d = src_dict.copy()
        traces = []
        _traces = d.pop("traces")
        for traces_item_data in _traces:
            traces_item = Trace.from_dict(traces_item_data)

            traces.append(traces_item)

        def _parse_client_version(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        client_version = _parse_client_version(d.pop("client_version", UNSET))

        def _parse_experiment_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        experiment_id = _parse_experiment_id(d.pop("experiment_id", UNSET))

        def _parse_log_stream_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        log_stream_id = _parse_log_stream_id(d.pop("log_stream_id", UNSET))

        _logging_method = d.pop("logging_method", UNSET)
        logging_method: Union[Unset, LogDataLoggingMethod]
        if isinstance(_logging_method, Unset):
            logging_method = UNSET
        else:
            logging_method = LogDataLoggingMethod(_logging_method)

        traces_ingest_request = cls(
            traces=traces,
            client_version=client_version,
            experiment_id=experiment_id,
            log_stream_id=log_stream_id,
            logging_method=logging_method,
        )

        traces_ingest_request.additional_properties = d
        return traces_ingest_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.dataset_created_at_sort import DatasetCreatedAtSort
    from ..models.dataset_last_edited_by_user_at_sort import DatasetLastEditedByUserAtSort
    from ..models.dataset_name_filter import DatasetNameFilter
    from ..models.dataset_name_sort import DatasetNameSort
    from ..models.dataset_project_last_used_at_sort import DatasetProjectLastUsedAtSort
    from ..models.dataset_projects_sort import DatasetProjectsSort
    from ..models.dataset_rows_sort import DatasetRowsSort
    from ..models.dataset_updated_at_sort import DatasetUpdatedAtSort
    from ..models.dataset_used_in_project_filter import DatasetUsedInProjectFilter


T = TypeVar("T", bound="ListDatasetParams")


@_attrs_define
class ListDatasetParams:
    """
    Attributes:
        filters (Union[Unset, list[Union['DatasetNameFilter', 'DatasetUsedInProjectFilter']]]):
        sort (Union['DatasetCreatedAtSort', 'DatasetLastEditedByUserAtSort', 'DatasetNameSort',
            'DatasetProjectLastUsedAtSort', 'DatasetProjectsSort', 'DatasetRowsSort', 'DatasetUpdatedAtSort', None, Unset]):
            Default: None.
    """

    filters: Union[Unset, list[Union["DatasetNameFilter", "DatasetUsedInProjectFilter"]]] = UNSET
    sort: Union[
        "DatasetCreatedAtSort",
        "DatasetLastEditedByUserAtSort",
        "DatasetNameSort",
        "DatasetProjectLastUsedAtSort",
        "DatasetProjectsSort",
        "DatasetRowsSort",
        "DatasetUpdatedAtSort",
        None,
        Unset,
    ] = None
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.dataset_created_at_sort import DatasetCreatedAtSort
        from ..models.dataset_last_edited_by_user_at_sort import DatasetLastEditedByUserAtSort
        from ..models.dataset_name_filter import DatasetNameFilter
        from ..models.dataset_name_sort import DatasetNameSort
        from ..models.dataset_project_last_used_at_sort import DatasetProjectLastUsedAtSort
        from ..models.dataset_projects_sort import DatasetProjectsSort
        from ..models.dataset_rows_sort import DatasetRowsSort
        from ..models.dataset_updated_at_sort import DatasetUpdatedAtSort

        filters: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.filters, Unset):
            filters = []
            for filters_item_data in self.filters:
                filters_item: dict[str, Any]
                if isinstance(filters_item_data, DatasetNameFilter):
                    filters_item = filters_item_data.to_dict()
                else:
                    filters_item = filters_item_data.to_dict()

                filters.append(filters_item)

        sort: Union[None, Unset, dict[str, Any]]
        if isinstance(self.sort, Unset):
            sort = UNSET
        elif isinstance(self.sort, DatasetNameSort):
            sort = self.sort.to_dict()
        elif isinstance(self.sort, DatasetCreatedAtSort):
            sort = self.sort.to_dict()
        elif isinstance(self.sort, DatasetUpdatedAtSort):
            sort = self.sort.to_dict()
        elif isinstance(self.sort, DatasetProjectLastUsedAtSort):
            sort = self.sort.to_dict()
        elif isinstance(self.sort, DatasetProjectsSort):
            sort = self.sort.to_dict()
        elif isinstance(self.sort, DatasetRowsSort):
            sort = self.sort.to_dict()
        elif isinstance(self.sort, DatasetLastEditedByUserAtSort):
            sort = self.sort.to_dict()
        else:
            sort = self.sort

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if filters is not UNSET:
            field_dict["filters"] = filters
        if sort is not UNSET:
            field_dict["sort"] = sort

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.dataset_created_at_sort import DatasetCreatedAtSort
        from ..models.dataset_last_edited_by_user_at_sort import DatasetLastEditedByUserAtSort
        from ..models.dataset_name_filter import DatasetNameFilter
        from ..models.dataset_name_sort import DatasetNameSort
        from ..models.dataset_project_last_used_at_sort import DatasetProjectLastUsedAtSort
        from ..models.dataset_projects_sort import DatasetProjectsSort
        from ..models.dataset_rows_sort import DatasetRowsSort
        from ..models.dataset_updated_at_sort import DatasetUpdatedAtSort
        from ..models.dataset_used_in_project_filter import DatasetUsedInProjectFilter

        d = src_dict.copy()
        filters = []
        _filters = d.pop("filters", UNSET)
        for filters_item_data in _filters or []:

            def _parse_filters_item(data: object) -> Union["DatasetNameFilter", "DatasetUsedInProjectFilter"]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    filters_item_type_0 = DatasetNameFilter.from_dict(data)

                    return filters_item_type_0
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                filters_item_type_1 = DatasetUsedInProjectFilter.from_dict(data)

                return filters_item_type_1

            filters_item = _parse_filters_item(filters_item_data)

            filters.append(filters_item)

        def _parse_sort(
            data: object,
        ) -> Union[
            "DatasetCreatedAtSort",
            "DatasetLastEditedByUserAtSort",
            "DatasetNameSort",
            "DatasetProjectLastUsedAtSort",
            "DatasetProjectsSort",
            "DatasetRowsSort",
            "DatasetUpdatedAtSort",
            None,
            Unset,
        ]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                sort_type_0_type_0 = DatasetNameSort.from_dict(data)

                return sort_type_0_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                sort_type_0_type_1 = DatasetCreatedAtSort.from_dict(data)

                return sort_type_0_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                sort_type_0_type_2 = DatasetUpdatedAtSort.from_dict(data)

                return sort_type_0_type_2
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                sort_type_0_type_3 = DatasetProjectLastUsedAtSort.from_dict(data)

                return sort_type_0_type_3
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                sort_type_0_type_4 = DatasetProjectsSort.from_dict(data)

                return sort_type_0_type_4
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                sort_type_0_type_5 = DatasetRowsSort.from_dict(data)

                return sort_type_0_type_5
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                sort_type_0_type_6 = DatasetLastEditedByUserAtSort.from_dict(data)

                return sort_type_0_type_6
            except:  # noqa: E722
                pass
            return cast(
                Union[
                    "DatasetCreatedAtSort",
                    "DatasetLastEditedByUserAtSort",
                    "DatasetNameSort",
                    "DatasetProjectLastUsedAtSort",
                    "DatasetProjectsSort",
                    "DatasetRowsSort",
                    "DatasetUpdatedAtSort",
                    None,
                    Unset,
                ],
                data,
            )

        sort = _parse_sort(d.pop("sort", UNSET))

        list_dataset_params = cls(filters=filters, sort=sort)

        list_dataset_params.additional_properties = d
        return list_dataset_params

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.project_db import ProjectDB
from ...models.project_type import ProjectType
from ...types import UNSET, Response, Unset


def _get_kwargs(user_id: str, *, project_type: Union[None, ProjectType, Unset] = UNSET) -> dict[str, Any]:
    params: dict[str, Any] = {}

    json_project_type: Union[None, Unset, str]
    if isinstance(project_type, Unset):
        json_project_type = UNSET
    elif isinstance(project_type, ProjectType):
        json_project_type = project_type.value
    else:
        json_project_type = project_type
    params["project_type"] = json_project_type

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {"method": "get", "url": f"/users/{user_id}/runs", "params": params}

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[HTTPValidationError, list["ProjectDB"]]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = ProjectDB.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[HTTPValidationError, list["ProjectDB"]]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    user_id: str, *, client: AuthenticatedClient, project_type: Union[None, ProjectType, Unset] = UNSET
) -> Response[Union[HTTPValidationError, list["ProjectDB"]]]:
    """Get Project Runs For User

     Get all user created runs across all projects they can access.

    This returns a filtered list of projects with only runs created by this user.

    Args:
        user_id (str):
        project_type (Union[None, ProjectType, Unset]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, list['ProjectDB']]]
    """

    kwargs = _get_kwargs(user_id=user_id, project_type=project_type)

    response = client.get_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


def sync(
    user_id: str, *, client: AuthenticatedClient, project_type: Union[None, ProjectType, Unset] = UNSET
) -> Optional[Union[HTTPValidationError, list["ProjectDB"]]]:
    """Get Project Runs For User

     Get all user created runs across all projects they can access.

    This returns a filtered list of projects with only runs created by this user.

    Args:
        user_id (str):
        project_type (Union[None, ProjectType, Unset]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, list['ProjectDB']]
    """

    return sync_detailed(user_id=user_id, client=client, project_type=project_type).parsed


async def asyncio_detailed(
    user_id: str, *, client: AuthenticatedClient, project_type: Union[None, ProjectType, Unset] = UNSET
) -> Response[Union[HTTPValidationError, list["ProjectDB"]]]:
    """Get Project Runs For User

     Get all user created runs across all projects they can access.

    This returns a filtered list of projects with only runs created by this user.

    Args:
        user_id (str):
        project_type (Union[None, ProjectType, Unset]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, list['ProjectDB']]]
    """

    kwargs = _get_kwargs(user_id=user_id, project_type=project_type)

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    user_id: str, *, client: AuthenticatedClient, project_type: Union[None, ProjectType, Unset] = UNSET
) -> Optional[Union[HTTPValidationError, list["ProjectDB"]]]:
    """Get Project Runs For User

     Get all user created runs across all projects they can access.

    This returns a filtered list of projects with only runs created by this user.

    Args:
        user_id (str):
        project_type (Union[None, ProjectType, Unset]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, list['ProjectDB']]
    """

    return (await asyncio_detailed(user_id=user_id, client=client, project_type=project_type)).parsed



import datetime
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.job_db_request_data import JobDBRequestData


T = TypeVar("T", bound="JobDB")


@_attrs_define
class JobDB:
    """
    Attributes:
        created_at (datetime.datetime):
        id (str):
        job_name (str):
        project_id (str):
        request_data (JobDBRequestData):
        retries (int):
        run_id (str):
        status (str):
        updated_at (datetime.datetime):
        completed_at (Union[None, Unset, datetime.datetime]):
        error_message (Union[None, Unset, str]):
        failed_at (Union[None, Unset, datetime.datetime]):
        migration_name (Union[None, Unset, str]):
        monitor_batch_id (Union[None, Unset, str]):
        processing_started (Union[None, Unset, datetime.datetime]):
        progress_message (Union[None, Unset, str]):
        progress_percent (Union[Unset, float]):  Default: 0.0.
        steps_completed (Union[Unset, int]):  Default: 0.
        steps_total (Union[Unset, int]):  Default: 0.
    """

    created_at: datetime.datetime
    id: str
    job_name: str
    project_id: str
    request_data: "JobDBRequestData"
    retries: int
    run_id: str
    status: str
    updated_at: datetime.datetime
    completed_at: Union[None, Unset, datetime.datetime] = UNSET
    error_message: Union[None, Unset, str] = UNSET
    failed_at: Union[None, Unset, datetime.datetime] = UNSET
    migration_name: Union[None, Unset, str] = UNSET
    monitor_batch_id: Union[None, Unset, str] = UNSET
    processing_started: Union[None, Unset, datetime.datetime] = UNSET
    progress_message: Union[None, Unset, str] = UNSET
    progress_percent: Union[Unset, float] = 0.0
    steps_completed: Union[Unset, int] = 0
    steps_total: Union[Unset, int] = 0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        created_at = self.created_at.isoformat()

        id = self.id

        job_name = self.job_name

        project_id = self.project_id

        request_data = self.request_data.to_dict()

        retries = self.retries

        run_id = self.run_id

        status = self.status

        updated_at = self.updated_at.isoformat()

        completed_at: Union[None, Unset, str]
        if isinstance(self.completed_at, Unset):
            completed_at = UNSET
        elif isinstance(self.completed_at, datetime.datetime):
            completed_at = self.completed_at.isoformat()
        else:
            completed_at = self.completed_at

        error_message: Union[None, Unset, str]
        if isinstance(self.error_message, Unset):
            error_message = UNSET
        else:
            error_message = self.error_message

        failed_at: Union[None, Unset, str]
        if isinstance(self.failed_at, Unset):
            failed_at = UNSET
        elif isinstance(self.failed_at, datetime.datetime):
            failed_at = self.failed_at.isoformat()
        else:
            failed_at = self.failed_at

        migration_name: Union[None, Unset, str]
        if isinstance(self.migration_name, Unset):
            migration_name = UNSET
        else:
            migration_name = self.migration_name

        monitor_batch_id: Union[None, Unset, str]
        if isinstance(self.monitor_batch_id, Unset):
            monitor_batch_id = UNSET
        else:
            monitor_batch_id = self.monitor_batch_id

        processing_started: Union[None, Unset, str]
        if isinstance(self.processing_started, Unset):
            processing_started = UNSET
        elif isinstance(self.processing_started, datetime.datetime):
            processing_started = self.processing_started.isoformat()
        else:
            processing_started = self.processing_started

        progress_message: Union[None, Unset, str]
        if isinstance(self.progress_message, Unset):
            progress_message = UNSET
        else:
            progress_message = self.progress_message

        progress_percent = self.progress_percent

        steps_completed = self.steps_completed

        steps_total = self.steps_total

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "created_at": created_at,
                "id": id,
                "job_name": job_name,
                "project_id": project_id,
                "request_data": request_data,
                "retries": retries,
                "run_id": run_id,
                "status": status,
                "updated_at": updated_at,
            }
        )
        if completed_at is not UNSET:
            field_dict["completed_at"] = completed_at
        if error_message is not UNSET:
            field_dict["error_message"] = error_message
        if failed_at is not UNSET:
            field_dict["failed_at"] = failed_at
        if migration_name is not UNSET:
            field_dict["migration_name"] = migration_name
        if monitor_batch_id is not UNSET:
            field_dict["monitor_batch_id"] = monitor_batch_id
        if processing_started is not UNSET:
            field_dict["processing_started"] = processing_started
        if progress_message is not UNSET:
            field_dict["progress_message"] = progress_message
        if progress_percent is not UNSET:
            field_dict["progress_percent"] = progress_percent
        if steps_completed is not UNSET:
            field_dict["steps_completed"] = steps_completed
        if steps_total is not UNSET:
            field_dict["steps_total"] = steps_total

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.job_db_request_data import JobDBRequestData

        d = src_dict.copy()
        created_at = isoparse(d.pop("created_at"))

        id = d.pop("id")

        job_name = d.pop("job_name")

        project_id = d.pop("project_id")

        request_data = JobDBRequestData.from_dict(d.pop("request_data"))

        retries = d.pop("retries")

        run_id = d.pop("run_id")

        status = d.pop("status")

        updated_at = isoparse(d.pop("updated_at"))

        def _parse_completed_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                completed_at_type_0 = isoparse(data)

                return completed_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        completed_at = _parse_completed_at(d.pop("completed_at", UNSET))

        def _parse_error_message(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        error_message = _parse_error_message(d.pop("error_message", UNSET))

        def _parse_failed_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                failed_at_type_0 = isoparse(data)

                return failed_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        failed_at = _parse_failed_at(d.pop("failed_at", UNSET))

        def _parse_migration_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        migration_name = _parse_migration_name(d.pop("migration_name", UNSET))

        def _parse_monitor_batch_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        monitor_batch_id = _parse_monitor_batch_id(d.pop("monitor_batch_id", UNSET))

        def _parse_processing_started(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                processing_started_type_0 = isoparse(data)

                return processing_started_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        processing_started = _parse_processing_started(d.pop("processing_started", UNSET))

        def _parse_progress_message(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        progress_message = _parse_progress_message(d.pop("progress_message", UNSET))

        progress_percent = d.pop("progress_percent", UNSET)

        steps_completed = d.pop("steps_completed", UNSET)

        steps_total = d.pop("steps_total", UNSET)

        job_db = cls(
            created_at=created_at,
            id=id,
            job_name=job_name,
            project_id=project_id,
            request_data=request_data,
            retries=retries,
            run_id=run_id,
            status=status,
            updated_at=updated_at,
            completed_at=completed_at,
            error_message=error_message,
            failed_at=failed_at,
            migration_name=migration_name,
            monitor_batch_id=monitor_batch_id,
            processing_started=processing_started,
            progress_message=progress_message,
            progress_percent=progress_percent,
            steps_completed=steps_completed,
            steps_total=steps_total,
        )

        job_db.additional_properties = d
        return job_db

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.embedding_data_meta_type_0 import EmbeddingDataMetaType0


T = TypeVar("T", bound="EmbeddingData")


@_attrs_define
class EmbeddingData:
    """
    Attributes:
        id (int):
        x (float):
        y (float):
        bbox (Union[None, Unset, list[float]]):
        bleu (Union[None, Unset, float]):
        cluster_id (Union[None, Unset, int]):
        confidence (Union[None, Unset, float]):
        data_error_potential (Union[None, Unset, float]):
        drift_score (Union[None, Unset, float]):
        error_type (Union[None, Unset, str]):
        generated_output (Union[None, Unset, str]):
        generated_uncertainty (Union[None, Unset, float]):
        gold (Union[None, Unset, str]):
        image (Union[None, Unset, str]):
        input_ (Union[None, Unset, str]):
        is_gold (Union[None, Unset, bool]):
        is_pred (Union[None, Unset, bool]):
        meta (Union['EmbeddingDataMetaType0', None, Unset]):
        perplexity (Union[None, Unset, float]):
        pred (Union[None, Unset, str]):
        rouge (Union[None, Unset, float]):
        span_end (Union[None, Unset, int]):
        span_start (Union[None, Unset, int]):
        target (Union[None, Unset, str]):
        text_sample (Union[None, Unset, str]):
    """

    id: int
    x: float
    y: float
    bbox: Union[None, Unset, list[float]] = UNSET
    bleu: Union[None, Unset, float] = UNSET
    cluster_id: Union[None, Unset, int] = UNSET
    confidence: Union[None, Unset, float] = UNSET
    data_error_potential: Union[None, Unset, float] = UNSET
    drift_score: Union[None, Unset, float] = UNSET
    error_type: Union[None, Unset, str] = UNSET
    generated_output: Union[None, Unset, str] = UNSET
    generated_uncertainty: Union[None, Unset, float] = UNSET
    gold: Union[None, Unset, str] = UNSET
    image: Union[None, Unset, str] = UNSET
    input_: Union[None, Unset, str] = UNSET
    is_gold: Union[None, Unset, bool] = UNSET
    is_pred: Union[None, Unset, bool] = UNSET
    meta: Union["EmbeddingDataMetaType0", None, Unset] = UNSET
    perplexity: Union[None, Unset, float] = UNSET
    pred: Union[None, Unset, str] = UNSET
    rouge: Union[None, Unset, float] = UNSET
    span_end: Union[None, Unset, int] = UNSET
    span_start: Union[None, Unset, int] = UNSET
    target: Union[None, Unset, str] = UNSET
    text_sample: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.embedding_data_meta_type_0 import EmbeddingDataMetaType0

        id = self.id

        x = self.x

        y = self.y

        bbox: Union[None, Unset, list[float]]
        if isinstance(self.bbox, Unset):
            bbox = UNSET
        elif isinstance(self.bbox, list):
            bbox = self.bbox

        else:
            bbox = self.bbox

        bleu: Union[None, Unset, float]
        if isinstance(self.bleu, Unset):
            bleu = UNSET
        else:
            bleu = self.bleu

        cluster_id: Union[None, Unset, int]
        if isinstance(self.cluster_id, Unset):
            cluster_id = UNSET
        else:
            cluster_id = self.cluster_id

        confidence: Union[None, Unset, float]
        if isinstance(self.confidence, Unset):
            confidence = UNSET
        else:
            confidence = self.confidence

        data_error_potential: Union[None, Unset, float]
        if isinstance(self.data_error_potential, Unset):
            data_error_potential = UNSET
        else:
            data_error_potential = self.data_error_potential

        drift_score: Union[None, Unset, float]
        if isinstance(self.drift_score, Unset):
            drift_score = UNSET
        else:
            drift_score = self.drift_score

        error_type: Union[None, Unset, str]
        if isinstance(self.error_type, Unset):
            error_type = UNSET
        else:
            error_type = self.error_type

        generated_output: Union[None, Unset, str]
        if isinstance(self.generated_output, Unset):
            generated_output = UNSET
        else:
            generated_output = self.generated_output

        generated_uncertainty: Union[None, Unset, float]
        if isinstance(self.generated_uncertainty, Unset):
            generated_uncertainty = UNSET
        else:
            generated_uncertainty = self.generated_uncertainty

        gold: Union[None, Unset, str]
        if isinstance(self.gold, Unset):
            gold = UNSET
        else:
            gold = self.gold

        image: Union[None, Unset, str]
        if isinstance(self.image, Unset):
            image = UNSET
        else:
            image = self.image

        input_: Union[None, Unset, str]
        if isinstance(self.input_, Unset):
            input_ = UNSET
        else:
            input_ = self.input_

        is_gold: Union[None, Unset, bool]
        if isinstance(self.is_gold, Unset):
            is_gold = UNSET
        else:
            is_gold = self.is_gold

        is_pred: Union[None, Unset, bool]
        if isinstance(self.is_pred, Unset):
            is_pred = UNSET
        else:
            is_pred = self.is_pred

        meta: Union[None, Unset, dict[str, Any]]
        if isinstance(self.meta, Unset):
            meta = UNSET
        elif isinstance(self.meta, EmbeddingDataMetaType0):
            meta = self.meta.to_dict()
        else:
            meta = self.meta

        perplexity: Union[None, Unset, float]
        if isinstance(self.perplexity, Unset):
            perplexity = UNSET
        else:
            perplexity = self.perplexity

        pred: Union[None, Unset, str]
        if isinstance(self.pred, Unset):
            pred = UNSET
        else:
            pred = self.pred

        rouge: Union[None, Unset, float]
        if isinstance(self.rouge, Unset):
            rouge = UNSET
        else:
            rouge = self.rouge

        span_end: Union[None, Unset, int]
        if isinstance(self.span_end, Unset):
            span_end = UNSET
        else:
            span_end = self.span_end

        span_start: Union[None, Unset, int]
        if isinstance(self.span_start, Unset):
            span_start = UNSET
        else:
            span_start = self.span_start

        target: Union[None, Unset, str]
        if isinstance(self.target, Unset):
            target = UNSET
        else:
            target = self.target

        text_sample: Union[None, Unset, str]
        if isinstance(self.text_sample, Unset):
            text_sample = UNSET
        else:
            text_sample = self.text_sample

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"id": id, "x": x, "y": y})
        if bbox is not UNSET:
            field_dict["bbox"] = bbox
        if bleu is not UNSET:
            field_dict["bleu"] = bleu
        if cluster_id is not UNSET:
            field_dict["cluster_id"] = cluster_id
        if confidence is not UNSET:
            field_dict["confidence"] = confidence
        if data_error_potential is not UNSET:
            field_dict["data_error_potential"] = data_error_potential
        if drift_score is not UNSET:
            field_dict["drift_score"] = drift_score
        if error_type is not UNSET:
            field_dict["error_type"] = error_type
        if generated_output is not UNSET:
            field_dict["generated_output"] = generated_output
        if generated_uncertainty is not UNSET:
            field_dict["generated_uncertainty"] = generated_uncertainty
        if gold is not UNSET:
            field_dict["gold"] = gold
        if image is not UNSET:
            field_dict["image"] = image
        if input_ is not UNSET:
            field_dict["input"] = input_
        if is_gold is not UNSET:
            field_dict["is_gold"] = is_gold
        if is_pred is not UNSET:
            field_dict["is_pred"] = is_pred
        if meta is not UNSET:
            field_dict["meta"] = meta
        if perplexity is not UNSET:
            field_dict["perplexity"] = perplexity
        if pred is not UNSET:
            field_dict["pred"] = pred
        if rouge is not UNSET:
            field_dict["rouge"] = rouge
        if span_end is not UNSET:
            field_dict["span_end"] = span_end
        if span_start is not UNSET:
            field_dict["span_start"] = span_start
        if target is not UNSET:
            field_dict["target"] = target
        if text_sample is not UNSET:
            field_dict["text_sample"] = text_sample

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.embedding_data_meta_type_0 import EmbeddingDataMetaType0

        d = src_dict.copy()
        id = d.pop("id")

        x = d.pop("x")

        y = d.pop("y")

        def _parse_bbox(data: object) -> Union[None, Unset, list[float]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                bbox_type_0 = cast(list[float], data)

                return bbox_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[float]], data)

        bbox = _parse_bbox(d.pop("bbox", UNSET))

        def _parse_bleu(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        bleu = _parse_bleu(d.pop("bleu", UNSET))

        def _parse_cluster_id(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        cluster_id = _parse_cluster_id(d.pop("cluster_id", UNSET))

        def _parse_confidence(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        confidence = _parse_confidence(d.pop("confidence", UNSET))

        def _parse_data_error_potential(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        data_error_potential = _parse_data_error_potential(d.pop("data_error_potential", UNSET))

        def _parse_drift_score(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        drift_score = _parse_drift_score(d.pop("drift_score", UNSET))

        def _parse_error_type(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        error_type = _parse_error_type(d.pop("error_type", UNSET))

        def _parse_generated_output(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        generated_output = _parse_generated_output(d.pop("generated_output", UNSET))

        def _parse_generated_uncertainty(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        generated_uncertainty = _parse_generated_uncertainty(d.pop("generated_uncertainty", UNSET))

        def _parse_gold(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        gold = _parse_gold(d.pop("gold", UNSET))

        def _parse_image(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        image = _parse_image(d.pop("image", UNSET))

        def _parse_input_(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        input_ = _parse_input_(d.pop("input", UNSET))

        def _parse_is_gold(data: object) -> Union[None, Unset, bool]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, bool], data)

        is_gold = _parse_is_gold(d.pop("is_gold", UNSET))

        def _parse_is_pred(data: object) -> Union[None, Unset, bool]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, bool], data)

        is_pred = _parse_is_pred(d.pop("is_pred", UNSET))

        def _parse_meta(data: object) -> Union["EmbeddingDataMetaType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                meta_type_0 = EmbeddingDataMetaType0.from_dict(data)

                return meta_type_0
            except:  # noqa: E722
                pass
            return cast(Union["EmbeddingDataMetaType0", None, Unset], data)

        meta = _parse_meta(d.pop("meta", UNSET))

        def _parse_perplexity(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        perplexity = _parse_perplexity(d.pop("perplexity", UNSET))

        def _parse_pred(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        pred = _parse_pred(d.pop("pred", UNSET))

        def _parse_rouge(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        rouge = _parse_rouge(d.pop("rouge", UNSET))

        def _parse_span_end(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        span_end = _parse_span_end(d.pop("span_end", UNSET))

        def _parse_span_start(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        span_start = _parse_span_start(d.pop("span_start", UNSET))

        def _parse_target(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        target = _parse_target(d.pop("target", UNSET))

        def _parse_text_sample(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        text_sample = _parse_text_sample(d.pop("text_sample", UNSET))

        embedding_data = cls(
            id=id,
            x=x,
            y=y,
            bbox=bbox,
            bleu=bleu,
            cluster_id=cluster_id,
            confidence=confidence,
            data_error_potential=data_error_potential,
            drift_score=drift_score,
            error_type=error_type,
            generated_output=generated_output,
            generated_uncertainty=generated_uncertainty,
            gold=gold,
            image=image,
            input_=input_,
            is_gold=is_gold,
            is_pred=is_pred,
            meta=meta,
            perplexity=perplexity,
            pred=pred,
            rouge=rouge,
            span_end=span_end,
            span_start=span_start,
            target=target,
            text_sample=text_sample,
        )

        embedding_data.additional_properties = d
        return embedding_data

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.split import Split
from ..types import UNSET, Unset

T = TypeVar("T", bound="SimilarToResponse")


@_attrs_define
class SimilarToResponse:
    """
    Attributes:
        from_split (Split):
        project_id (str):
        run_id (str):
        similar_ids (list[int]):
        split (Split):
        from_inference_name (Union[None, Unset, str]):
        inference_name (Union[None, Unset, str]):
        task (Union[None, Unset, str]):
    """

    from_split: Split
    project_id: str
    run_id: str
    similar_ids: list[int]
    split: Split
    from_inference_name: Union[None, Unset, str] = UNSET
    inference_name: Union[None, Unset, str] = UNSET
    task: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from_split = self.from_split.value

        project_id = self.project_id

        run_id = self.run_id

        similar_ids = self.similar_ids

        split = self.split.value

        from_inference_name: Union[None, Unset, str]
        if isinstance(self.from_inference_name, Unset):
            from_inference_name = UNSET
        else:
            from_inference_name = self.from_inference_name

        inference_name: Union[None, Unset, str]
        if isinstance(self.inference_name, Unset):
            inference_name = UNSET
        else:
            inference_name = self.inference_name

        task: Union[None, Unset, str]
        if isinstance(self.task, Unset):
            task = UNSET
        else:
            task = self.task

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "from_split": from_split,
                "project_id": project_id,
                "run_id": run_id,
                "similar_ids": similar_ids,
                "split": split,
            }
        )
        if from_inference_name is not UNSET:
            field_dict["from_inference_name"] = from_inference_name
        if inference_name is not UNSET:
            field_dict["inference_name"] = inference_name
        if task is not UNSET:
            field_dict["task"] = task

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        from_split = Split(d.pop("from_split"))

        project_id = d.pop("project_id")

        run_id = d.pop("run_id")

        similar_ids = cast(list[int], d.pop("similar_ids"))

        split = Split(d.pop("split"))

        def _parse_from_inference_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        from_inference_name = _parse_from_inference_name(d.pop("from_inference_name", UNSET))

        def _parse_inference_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        inference_name = _parse_inference_name(d.pop("inference_name", UNSET))

        def _parse_task(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        task = _parse_task(d.pop("task", UNSET))

        similar_to_response = cls(
            from_split=from_split,
            project_id=project_id,
            run_id=run_id,
            similar_ids=similar_ids,
            split=split,
            from_inference_name=from_inference_name,
            inference_name=inference_name,
            task=task,
        )

        similar_to_response.additional_properties = d
        return similar_to_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
